/****************************************************************************/
/* IPMON version 1.0                                                        */
/****************************************************************************/
/* You can redistribute this program as much as you want.  All I ask is     */
/* that you give credit where credit is due.  If you make modifications or  */
/* bug fixes, please send them to me so they can be incorporated into the   */
/* next release.                                                            */
/*                                                                          */
/* IPMON takes its basic packet processing shell from NFSWATCH.  Much       */
/* thanks to the creators of NFSWATCH:                                      */
/*                                                                          */
/* Dave Curry					Jeff Mogul                  */
/* SRI International				Digital Equipment Corp.     */
/* 333 Ravenswood Avenue			Western Research Laboratory */
/* Menlo Park, CA 94025				100 Hamilton Avenue         */
/* davy@erg.sri.com				Palo Alto, CA 94301         */
/* 						mogul@decwrl.dec.com        */
/*                                                                          */
/* For a good set of base code for me to hack and build on.                 */
/*                                                                          */
/* Randall Stewart                                                          */
/* Fujitsu Network Switching                                                */
/* 4403 Bland Road                                                          */
/* Raleigh, N.C. 27609                                                      */
/* rrs%fns-nc1%mcnc-ort@uunet.uu.net                                        */
/****************************************************************************/
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <net/if.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <net/nit_if.h>
#include <net/nit_buf.h>

char	*devices[] = {
	"le0", "ie0", "le1", "ie1", "le2", "ie2",
	"le3", "ie3", "le4", "ie4",
	0
};
FILE *flogging;
#include "ipmon.h"
#include "externs.h"
int pip[2];
char *inputBuf;
extern char *optarg;
extern int optind,opterr;
int statistics;
#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 64
#endif

#ifndef MAXHOSTADDR
#define MAXHOSTADDR 8
#endif

char		*pname;				/* program name		*/
Counter byteTotals[3];
Counter timeFlush[3];
Counter udpCnts[MAXNUMOFPORTS][2];
Counter tcpCnts[MAXNUMOFPORTS][2];
Counter tcpTotal[2];
Counter udpTotal[2];
Counter icmpTotal[2];
Counter broadcast[2];
Counter pkt_total[2];
Counter unknownip[2];
Counter netdisk[2];
Counter icmpCnts[MAXICMPMSG][2];
Counter arpCnts[2];
Counter revarpCnts[2];
Counter unknown[2];
Counter	pkt_drops[2];			/* total packets dropped*/
PacketCounter *pktUdp;
PacketCounter *pktTcp;
char logFileName[128];
FILE		*logfp;				/* log file pointer	*/
int *totMonLost,*showTotals;
int mumMode;
char *strings[20];
/************************/
/* shared memory stuff  */
int *packetOfInterest;
int *tcpFlag;
char **foo;

/************************/

int		if_fd[MAXINTERFACES];		/* nit device file desc	*/
int		logging = 0;			/* 1 when logging on	*/
int		do_update = 0;			/* time to update screen*/
int		cycletime = CYCLETIME;		/* update cycle time	*/
int		scanCycle = 6;
int		truncation = 200;		/* pkt trunc len - magic*/
int		screen_inited = 0;		/* 1 when in curses	*/
int 		advanceFlag;
int		silent;				/* quiet to screen option */
struct timeval	starttime;			/* time we started	*/

int		ninterfaces;			/* number of interfaces	*/

u_long		thisdst = 0;			/* cached IP dst of pkt	*/
u_long		srcaddrs[MAXHOSTADDR];		/* src host net addrs	*/
u_long		dstaddrs[MAXHOSTADDR];		/* dst host net addrs	*/

char		myhost[MAXHOSTNAMELEN];		/* local host name	*/
char		srchost[MAXHOSTNAMELEN];	/* source host name	*/
char		dsthost[MAXHOSTNAMELEN];	/* destination host name*/

char		*prompt = PROMPT;		/* prompt string	*/
char		*filelist = NULL;		/* list of files	*/
int fk;

main(argc, argv)
int argc;
char **argv;
{
	register int i;
	struct sigvec sv;
	char *device;
	extern void finish(), ipmon();

	device = NULL;
	pname = *argv;
	logging = 0;
	statistics = 0;
	mumMode = 0;
	/*
	 * Get our host name.  The default destination
	 * host is the one we're running on.
	 */
	if (gethostname(myhost, sizeof(myhost)) < 0) {
		error("gethostname");
		finish(-1);
	}
	flogging = NULL;
	silent = 0;
	logfp = NULL;
	while((i = getopt(argc,argv,"l:c:s:?qd:S:bm")) != -1){
		switch (i){
			case 'm':
				mumMode = 1;
				break;
			case 'b':
				statistics = 1;
				break;
			case 'S':
				flogging = fopen(optarg,"w+");
				break;
			case 'd':
				device = optarg;
				break;
			case 'q':
				silent = 1;
				break;
			case 'l':
				strcpy(logFileName,optarg);
				logfp = fopen(optarg,"a+");
				if(logfp == NULL){
					perror("Can't open logfile");
					exit(0);
				}
				break;
			case 'c':
				cycletime = strtol(optarg,NULL,0);
				if(cycletime < 1){
					fprintf(stderr,"Cycle time must be at least 1\n");
					exit(0);
				}
				break;
			case 's':
				scanCycle = strtol(optarg,NULL,0);
				if(scanCycle < 1){
					fprintf(stderr,"Cycle time must be at least 1\n");
					exit(0);
				}
				break;
			case '?':
			default:
				useage(argv[0]);
				exit(0);
		}
	}
	
	/*
	 * Trap signals so we can clean up.
	 */

	sv.sv_handler = finish;
	sv.sv_mask = sv.sv_flags = 0;
	get_semy();
	put_semy();
	(void) sigvec(SIGINT, &sv, (struct sigvec *) 0);
	(void) sigvec(SIGQUIT, &sv, (struct sigvec *) 0);
	sv.sv_handler = SIG_IGN;
	(void) sigvec(SIGABRT, &sv, (struct sigvec *) 0);

	/*
	 * Set up the network interface tap right away,
	 * since we probably need super-user permission.
	 */
	setupShm();



	ninterfaces = 0;
	if_fd[ninterfaces] = setup_nit_dev(&device);
		if (if_fd[ninterfaces] >= 0)
			ninterfaces++;

	if (ninterfaces < 1) {
		fprintf(stderr, "%s: no valid interfaces.\n", pname);
		finish(-1);
	}

	/*
	 * Now lose super-user permission, since we
	 * don't need it for anything else.
	 */
	(void) setreuid(getuid(), getuid());
	/*
	 * Set up the screen.
	 */
	setup_pkt_counters();
	setup_screen(device);
	if(pipe(pip) == -1){
		perror("Can't setup pipe");
		exit(-1);
	}
	fk = fork();
	if(fk == -1){
		perror("Can't fork");
		exit(0);
	}
	if(fk > 0){
		iploop();
		exit(0);
	}
	/*
	 * Go watch packets.  Never returns.
	 */
	/* child does this */
	ipmon();
}

/*
 * ipmon - main packet reading loop.
 */
void
ipmon()
{
	int oldmask;
	int i, cc;
	char *malloc();
	fd_set readfds;
	struct sigvec sv;
	extern void wakeup();
	struct itimerval itv;
	register char *bp, *cp, *bufstop;
	int tdrops[MAXINTERFACES];
	struct nit_bufhdr *hdrp;
	struct nit_ifdrops *ndp;

	timeFlush[0] = timeFlush[1] = timeFlush[2] = time(0);

	/*
	 * Set up the alarm handler.
	 */
	sv.sv_handler = wakeup;
	sv.sv_mask = sv.sv_flags = 0;

	(void) sigvec(SIGALRM, &sv, (struct sigvec *) 0);

	sv.sv_handler = reDraw;
	(void) sigvec(SIGUSR2, &sv, (struct sigvec *) 0);

	sv.sv_handler = finish;
	(void) sigvec(SIGUSR1, &sv, (struct sigvec *) 0);
	/*
	 * Set up the alarm clock.
	 */
	(void) bzero((char *) &itv, sizeof(struct itimerval));

	itv.it_interval.tv_sec = cycletime;
	itv.it_interval.tv_usec = 0;

	itv.it_value = itv.it_interval;

	(void) setitimer(ITIMER_REAL, &itv, (struct itimerval *) 0);

	/*
	 * Set the start time.
	 */
	(void) gettimeofday(&starttime, (struct timezone *) 0);

	/*
	 * Flush the read queue of any packets that accumulated
	 * during setup time.
	 */
	for (i=0; i < ninterfaces; i++) {
		flush_nit(if_fd[i]);
		tdrops[i] = 0;
	}

	for (;;) {
		FD_ZERO(&readfds);

		for (i=0; i < ninterfaces; i++)
			FD_SET(if_fd[i], &readfds);

		/*
		 * See which nets have packets to read.
		 */
		cc = select(NFDBITS, &readfds, (fd_set *) 0, (fd_set *) 0, 0);

		if ((cc < 0) && (errno != EINTR)) {
			error("select");
			finish(-1);
		}
		if (cc == 0) {
			continue;
		}
		
		/*
		 * For each interface...
		 */
		for (i=0; i < ninterfaces; i++) {
			/*
			 * Nothing to read.
			 */
			if (!FD_ISSET(if_fd[i], &readfds))
				continue;

			/*
			 * Now read packets from the nit device.
			 */
			if ((cc = read(if_fd[i], bufInHead, NIT_CHUNKSIZE)) <= 0)
				continue;

			bufstop = bufInHead + cc;
			bp = bufInHead;
			advanceFlag = 0;
			/*
			 * Loop through the chunk, extracting packets.
			 */
			while (bp < bufstop) {
				cp = bp;

				/*
				 * Get the nit header.
				 */
				hdrp = (struct nit_bufhdr *) cp;
				cp += sizeof(struct nit_bufhdr);

				/*
				 * Get the number of dropped packets.
				 */
				ndp = (struct nit_ifdrops *) cp;
				cp += sizeof(struct nit_ifdrops);

				/* compensate at least header and */
				/* trailers of lost packets */

				oldmask = sigblock(0x7fffffff);
				if((flogging != NULL) && ((ndp->nh_drops-tdrops[i]) > 0)){
					fprintf(flogging,"%d:\n",((ndp->nh_drops-tdrops[i]) * 12));
				}
				byteTotals[0] += ((ndp->nh_drops-tdrops[i]) * 12); 
				byteTotals[1] += ((ndp->nh_drops-tdrops[i]) * 12); 
				byteTotals[2] += ((ndp->nh_drops-tdrops[i]) * 12); 
				sigsetmask(oldmask);
				pkt_drops[INTERVAL] += ndp->nh_drops - tdrops[i];
				pkt_drops[SUMTOTAL] += ndp->nh_drops - tdrops[i];
				tdrops[i] = ndp->nh_drops;

				/*
				 * Filter the packet.
				 */
				pkt_filter(cp, hdrp->nhb_msglen);

				/*
				 * Skip over this packet.
				 */
				bp += hdrp->nhb_totlen;
			}
		}
		if(advanceFlag){
			bufInHead += NIT_CHUNKSIZE;
			if(bufInHead >= bufInEND)
				bufInHead = bufInTOP;
			/* notify child */
			write(pip[1],(char*)&cc,(sizeof(int)));
		}
	}
}

void
iploop()
{
	int cc,bytesIn[NUMOFBUFS],numIn;
	char dispBuf[64];
	long lost;
	register int i,len;
	fd_set readfds;
	struct sigvec sv;
	struct itimerval itv;

	sv.sv_handler = scanLoop;
	sv.sv_mask = sv.sv_flags = 0;
	(void) sigvec(SIGALRM, &sv, (struct sigvec *) 0);
	sv.sv_handler = flushFile;
	(void) sigvec(SIGABRT, &sv, (struct sigvec *) 0);

	/* set no delay */
	fcntl(pip[0],F_SETFL,FNDELAY);

	for (;;) {
		FD_ZERO(&readfds);

		FD_SET(0, &readfds);
		FD_SET(pip[0],&readfds);
		/*
		 * See which nets have packets to read.
		 */
		cc = select(NFDBITS, &readfds, (fd_set *) 0, (fd_set *) 0, NULL);

		if ((cc < 0) && (errno != EINTR)) {
			error("select");
			finish(-1);
		}
		if ((cc > 0) && FD_ISSET(pip[0], &readfds)){
			numIn = read(pip[0],(char *)bytesIn,(sizeof(int)*NUMOFBUFS));
			if(numIn > 0){
				len = numIn / sizeof(int);
				/* lost some data */
				bufInTail += (NIT_CHUNKSIZE * (len-1));
				if(bufInTail >= bufInEND){
					lost = (long)bufInTail - (long)bufInEND;
					bufInTail = bufInTOP + lost;
				}
				if(len > 1){
					*totMonLost += (len-1);
					sprintf(dispBuf,"Lost %d buffers",(len-1));
					showTotalLost();
					showError(dispBuf);
				}
				displayPkts(bytesIn[(len-1)]);
			}
		}
		if ((cc > 0) && FD_ISSET(0, &readfds)){
			clearError();
			clearHelp();
			command();
		}
	}
}

void
flushFile()
{
	if(logfp != NULL)
		fflush(logfp);
	return;
}
