/****************************************************************************/
/* IPMON version 1.0                                                        */
/****************************************************************************/
/* You can redistribute this program as much as you want.  All I ask is     */
/* that you give credit where credit is due.  If you make modifications or  */
/* bug fixes, please send them to me so they can be incorporated into the   */
/* next release.                                                            */
/*                                                                          */
/* IPMON takes its basic packet processing shell from NFSWATCH.  Much       */
/* thanks to the creators of NFSWATCH:                                      */
/*                                                                          */
/* Dave Curry					Jeff Mogul                  */
/* SRI International				Digital Equipment Corp.     */
/* 333 Ravenswood Avenue			Western Research Laboratory */
/* Menlo Park, CA 94025				100 Hamilton Avenue         */
/* davy@erg.sri.com				Palo Alto, CA 94301         */
/* 						mogul@decwrl.dec.com        */
/*                                                                          */
/* For a good set of base code for me to hack and build on.                 */
/*                                                                          */
/* Randall Stewart                                                          */
/* Fujitsu Network Switching                                                */
/* 4403 Bland Road                                                          */
/* Raleigh, N.C. 27609                                                      */
/* rrs%fns-nc1%mcnc-ort@uunet.uu.net                                        */
/****************************************************************************/
#ifdef sun
#include <sys/param.h>
#include <sys/stropts.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/file.h>
#include <net/if.h>
#include <signal.h>
#include <stdio.h>

#include <net/nit_if.h>
#include <net/nit_buf.h>

#include "ipmon.h"
#include "externs.h"

#if NOFILE > 64
#define SUNOS40 1
#endif /* NOFILE */

/*
 * setup_nit_dev - set up the network interface tap.
 */
int
setup_nit_dev(device)
char **device;
{
	int s, fd;
	u_int chunksz;
	char *strdup();
	u_long if_flags;
	char buf[BUFSIZ];
	struct ifreq ifr;
	struct ifconf ifc;
	struct strioctl si;
	struct timeval timeout;

	/*
	 * If the interface device was not specified,
	 * get the default one.
	 */
	if (*device == NULL) {
		/*
		 * Grab a socket.
		 */
		if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
			error("socket");
			finish(-1);
		}

		ifc.ifc_buf = buf;
		ifc.ifc_len = sizeof(buf);

		/*
		 * See what device it's attached to.
		 */
		if (ioctl(s, SIOCGIFCONF, (char *) &ifc) < 0) {
			error("ioctl: SIOCGIFCONF");
			finish(-1);
		}

		*device = strdup(ifc.ifc_req->ifr_name);
		(void) close(s);
	}

	/*
	 * We want the ethernet in promiscuous mode, and
	 * we want to know about dropped packets.
	 */
	if_flags = NI_DROPS | NI_PROMISC;

	/*
	 * Open the network interface tap.
	 */
	if ((fd = open(NIT_DEV, O_RDONLY)) < 0) {
		error("nit: open");
		finish(-1);
	}

	/*
	 * Arrange to get discrete messages.
	 */
	if (ioctl(fd, I_SRDOPT, (char *) RMSGD) < 0) {
		error("ioctl: I_SRDOPT");
		finish(-1);
	}

	/*
	 * Push and configure the nit buffering module.
	 */
	if (ioctl(fd, I_PUSH, NIT_BUF) < 0) {
		error("ioctl: I_PUSH NIT_BUF");
		finish(-1);
	}

	/*
	 * Set the read timeout.
	 */
	timeout.tv_sec = 1;
	timeout.tv_usec = 0;

	si.ic_cmd = NIOCSTIME;
	si.ic_timout = INFTIM;
	si.ic_len = sizeof(timeout);
	si.ic_dp = (char *) &timeout;

	if (ioctl(fd, I_STR, (char *) &si) < 0) {
		error("ioctl: I_STR NIOCSTIME");
		finish(-1);
	}

	/*
	 * Set the chunk size.
	 */
	chunksz = NIT_CHUNKSIZE;
	si.ic_cmd = NIOCSCHUNK;
	si.ic_len = sizeof(chunksz);
	si.ic_dp = (char *) &chunksz;

	if (ioctl(fd, I_STR, (char *) &si) < 0) {
		error("ioctl: I_STR NIOCSCHUNK");
		finish(-1);
	}

	/*
	 * Configure the network interface tap by binding it
	 * to the underlying interface, setting the snapshot
	 * length, and setting the flags.
	 */
	(void) strncpy(ifr.ifr_name, *device, sizeof(ifr.ifr_name));
	ifr.ifr_name[sizeof(ifr.ifr_name)-1] = '\0';

	si.ic_cmd = NIOCBIND;
	si.ic_len = sizeof(ifr);
	si.ic_dp = (char *) &ifr;

	/*
	 * If the bind fails, there's no such device.
	 */
	if (ioctl(fd, I_STR, (char *) &si) < 0) {
		close(fd);
		return(-1);
	}

	/*
	 * SNAP is buggy on SunOS 4.0.x
	 */
#ifndef SUNOS40
	si.ic_cmd = NIOCSSNAP;
	si.ic_len = sizeof(truncation);
	si.ic_dp = (char *) &truncation;

	if (ioctl(fd, I_STR, (char *) &si) < 0) {
		error("ioctl: I_STR NIOCSSNAP");
		finish(-1);
	}
#endif /* SUNOS40 */

	si.ic_cmd = NIOCSFLAGS;
	si.ic_len = sizeof(if_flags);
	si.ic_dp = (char *) &if_flags;

	if (ioctl(fd, I_STR, (char *) &si) < 0) {
		error("ioctl: I_STR NIOCSFLAGS");
		finish(-1);
	}

	return(fd);
}

/*
 * flush_nit - flush data from the nit.
 */
void
flush_nit(fd)
int fd;
{
	if (ioctl(fd, I_FLUSH, (char *) FLUSHR) < 0) {
		error("ioctl: I_FLUSH");
		finish(-1);
	}
}
#endif /* sun */
