/****************************************************************************/
/* IPMON version 1.0                                                        */
/****************************************************************************/
/* You can redistribute this program as much as you want.  All I ask is     */
/* that you give credit where credit is due.  If you make modifications or  */
/* bug fixes, please send them to me so they can be incorporated into the   */
/* next release.                                                            */
/*                                                                          */
/* IPMON takes its basic packet processing shell from NFSWATCH.  Much       */
/* thanks to the creators of NFSWATCH:                                      */
/*                                                                          */
/* Dave Curry					Jeff Mogul                  */
/* SRI International				Digital Equipment Corp.     */
/* 333 Ravenswood Avenue			Western Research Laboratory */
/* Menlo Park, CA 94025				100 Hamilton Avenue         */
/* davy@erg.sri.com				Palo Alto, CA 94301         */
/* 						mogul@decwrl.dec.com        */
/*                                                                          */
/* For a good set of base code for me to hack and build on.                 */
/*                                                                          */
/* Randall Stewart                                                          */
/* Fujitsu Network Switching                                                */
/* 4403 Bland Road                                                          */
/* Raleigh, N.C. 27609                                                      */
/* rrs%fns-nc1%mcnc-ort@uunet.uu.net                                        */
/****************************************************************************/
#include <sys/param.h>
#include <sys/socket.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>
#include <netinet/ip_var.h>
#include <netinet/udp_var.h>
#ifdef sun
#  define NND	1
#  include <sun/ndio.h>
#endif /* sun */
#include <stdio.h>

#include "ipmon.h"
#include "externs.h"
#include "screen.h"

/*
 * Ethernet broadcast address.
 */
static	struct ether_addr ether_broadcast = {
	0xff, 0xff, 0xff, 0xff, 0xff, 0xff
};

/*
 * pkt_filter - count a packet, and pass it off to the appropriate filter.
 */
void
pkt_filter(cp, length)
u_int length;
char *cp;
{
	struct ip *ip;
	int packet[PACKETSIZE];
	register int bdcst,bytecnt;
	struct ether_header eheader;
	int oldmask;

	/*
	 * Count this packet in the network totals.
	 */
	pkt_total[INTERVAL]++;
	pkt_total[SUMTOTAL]++;
	/*
	 * Extract the ethernet header.
	 */
	(void) bcopy(cp, (char *) &eheader, sizeof(struct ether_header));
	(void) bcopy(cp + sizeof(struct ether_header), (char *) packet,
		(int) (length - sizeof(struct ether_header)));

	/*
	 * See if it's a broadcast packet, and count it if it is.
	 */
	bdcst = !bcmp((char *) &eheader.ether_dhost, (char *) &ether_broadcast,
			sizeof(struct ether_addr));
	if(bdcst){
		broadcast[INTERVAL]++;
		broadcast[SUMTOTAL]++;
	}
	/*
	 * Figure out what kind of packet it is, and pass
	 * it off to the appropriate filter.
	 */
	/* calculate the data size + eheader + preamble + trailer (802.3 assumed )*/
	oldmask = sigblock(0x7fffffff);
	bytecnt = length + sizeof(eheader) + 8 + 4;
	if(flogging != NULL)
		fprintf(flogging,"%d\n",bytecnt);



	byteTotals[0] += bytecnt;
	byteTotals[1] += bytecnt;
	byteTotals[2] += bytecnt;
	sigsetmask(oldmask);

	switch (ntohs(eheader.ether_type)) {
	case ETHERTYPE_IP:		/* IP packet			*/
		ip = (struct ip *) packet;
		/*
		 * If we want this packet, count it in the host
		 * totals and pass it off.
		 */
		ip_filter(ip, length, ip->ip_src.s_addr,
			ip->ip_dst.s_addr);
		break;
	case ETHERTYPE_ARP:		/* Address Resolution Protocol	*/
		/*
		 * If we want this packet, count it in the host
		 * totals and then count it in the packet
		 * type counters.
		 */
		arpCnts[INTERVAL]++;
		arpCnts[SUMTOTAL]++;
		break;
	case ETHERTYPE_REVARP:		/* Reverse Addr Resol Protocol	*/
		/*
		 * If we want this packet, count it in the host
		 * totals and then count it in the packet
		 * type counters.
		 */
		revarpCnts[INTERVAL]++;
		revarpCnts[SUMTOTAL]++;
		if(*tcpFlag == REVARP_VALUE){
			advanceFlag++;
		}
		break;
#ifdef notdef
	case ETHERTYPE_PUP:		/* Xerox PUP			*/
#endif /* notdef */
	default:
		if(*tcpFlag == UNKNOWN_VALUE){
			advanceFlag++;
		}
		unknown[INTERVAL]++;			/* who knows...			*/
		unknown[SUMTOTAL]++;
		break;
	}
}

/*
 * ip_filter - strip off the IP header and pass off to the appropriate
 *	       filter.
 */
void
ip_filter(ip, length, src, dst)
register struct ip *ip;
u_long src, dst;
u_int length;
{
	register int *data;
	register int datalength;

	data = (int *) ip;
	data += ip->ip_hl;
	datalength = ntohs(ip->ip_len) - (4 * ip->ip_hl);

	/*
	 * Figure out what kind of IP packet this is, and
	 * pass it off to the appropriate filter.
	 */
	switch (ip->ip_p) {
	case IPPROTO_TCP:		/* transmission control protocol*/
		tcpTotal[INTERVAL]++;
		tcpTotal[SUMTOTAL]++;
		tcp_filter((struct tcphdr *) data, datalength,
			src, dst);
		break;
	case IPPROTO_UDP:		/* user datagram protocol	*/
		udpTotal[INTERVAL]++;
		udpTotal[SUMTOTAL]++;
		udp_filter((struct udphdr *) data, datalength,
			src, dst);
		break;
	case IPPROTO_ND:		/* Sun Network Disk protocol	*/
		netdisk[INTERVAL]++;
		netdisk[SUMTOTAL]++;
		break;
	case IPPROTO_ICMP:		/* control message protocol	*/
		icmpTotal[INTERVAL]++;
		icmpTotal[SUMTOTAL]++;
		icmp_filter((struct icmp *) data, datalength,
			src, dst);
		break;
	default:
		if(*tcpFlag == UNKNOWNIP_VALUE){
			advanceFlag++;
		}
		unknownip[INTERVAL]++;			/* who knows...			*/
		unknownip[SUMTOTAL]++;
		break;
	}
}

/*
 * tcp_filter - count TCP packets.
 */
void
tcp_filter(tcp, length, src, dst)
register struct tcphdr *tcp;
u_long src, dst;
u_int length;
{
	/*
	 * Count the packet.
	 */
	if((long)(tcp->th_dport) < MAXNUMOFPORTS){
		tcpCnts[tcp->th_dport][INTERVAL]++;
		tcpCnts[tcp->th_dport][SUMTOTAL]++;
	}
	if((*tcpFlag == TCPVALUE) && 
	((tcp->th_dport == *packetOfInterest) || (tcp->th_sport == *packetOfInterest)))
		advanceFlag++;
}

/*
 * udp_filter - count UDP packets, pass RPC packets to the RPC filter.
 */
void
udp_filter(udp, length, src, dst)
register struct udphdr *udp;
u_long src, dst;
u_int length;
{
	/*
	 * Count as a UDP packet.
	 */
	if((long)(udp->uh_dport) < MAXNUMOFPORTS){
		udpCnts[udp->uh_dport][INTERVAL]++;
		udpCnts[udp->uh_dport][SUMTOTAL]++;
	}
	if((*tcpFlag == UDPVALUE) &&
	(udp->uh_dport == *packetOfInterest) || (udp->uh_sport == *packetOfInterest))
		advanceFlag++;
}

/*
 * icmp_filter - count ICMP packets.
 */
void icmp_filter(icp, length, src, dst)
register struct icmp *icp;
u_long src, dst;
u_int length;
{
	if((long)(icp->icmp_type) < MAXICMPMSG){
		icmpCnts[icp->icmp_type][INTERVAL]++;
		icmpCnts[icp->icmp_type][SUMTOTAL]++;
	}
	if((*tcpFlag == ICMPVALUE) && 
	(icp->icmp_type == *packetOfInterest))
		advanceFlag++;
}

void
showPkt(cp,length)
u_int length;
char *cp;
{
	struct ip *ip;
	int packet[PACKETSIZE];
	register int bdcst;
	struct ether_header eheader;

	/*
	 * Extract the ethernet header.
	 */

	(void) bcopy(cp, (char *) &eheader, sizeof(struct ether_header));
	(void) bcopy(cp + sizeof(struct ether_header), (char *) packet,
		(int) (length - sizeof(struct ether_header)));

	/*
	 * See if it's a broadcast packet, and count it if it is.
	 */
	bdcst = !bcmp((char *) &eheader.ether_dhost, (char *) &ether_broadcast,
			sizeof(struct ether_addr));
	/*
	 * Figure out what kind of packet it is, and pass
	 * it off to the appropriate filter.
	 */
	switch (ntohs(eheader.ether_type)) {
		case ETHERTYPE_IP:		/* IP packet			*/
			ip = (struct ip *) packet;
			/*
			 * If we want this packet, count it in the host
			 * totals and pass it off.
			 */
			ShowIP(ip, length, ip->ip_src.s_addr,ip->ip_dst.s_addr,&eheader);
			break;
		case ETHERTYPE_ARP:		/* Address Resolution Protocol	*/
		case ETHERTYPE_REVARP:		/* Reverse Addr Resol Protocol	*/
		default:
						/* don't care about any but ip */
			if((*tcpFlag == UNKNOWN_VALUE) || 
				(*tcpFlag == REVARP_VALUE))
				ShowUnknown(&eheader,packet,(length - sizeof(struct ether_header)));
			break;
	}
}

void
ShowIP(ip, length, src, dst,eheader)
register struct ip *ip;
u_long src, dst;
u_int length;
struct ether_header *eheader;
{
	register int *data;
	register int datalength;

	data = (int *) ip;
	data += ip->ip_hl;
	datalength = ntohs(ip->ip_len) - (4 * ip->ip_hl);

	/*
	 * Figure out what kind of IP packet this is, and
	 * pass it off to the appropriate filter.
	 */
	switch (ip->ip_p) {
	case IPPROTO_TCP:		/* transmission control protocol*/
		if(*tcpFlag == TCPVALUE)
			showTcp((struct tcphdr *) data, datalength,
			src, dst);
		break;
	case IPPROTO_UDP:		/* user datagram protocol	*/
		if(*tcpFlag == UDPVALUE)
			showUdp((struct udphdr *) data, datalength,
			src, dst);
		break;
	case IPPROTO_ICMP:
		if(*tcpFlag == ICMPVALUE){
			showIcmp((struct icmp *) data, datalength,
				src, dst,eheader);
		}
		break;
	default:
		if(*tcpFlag == UNKNOWN_VALUE){
			showIPUnknown(ip,data,datalength,src,dst);
		}
		/* don't care about other IP packets */
		break;
	}
}

void
ShowUnknown(eheader, packet, length)
struct ether_header *eheader;
char *packet;
int length;
{
	showEnet(eheader->ether_type,eheader->ether_dhost.ether_addr_octet,
		eheader->ether_shost.ether_addr_octet,packet,length);
	return;
}

void
showIPUnknown(ip, data, length, src, dst)
register struct ip *ip;
int *data;
u_long src, dst;
u_int length;
{
	int ip_v,ip_hl;
	
	ip_v = ip->ip_v;
	ip_hl = ip->ip_hl;

	showJunk(ip_v,ip_hl,ip->ip_tos,ip->ip_id,ip->ip_p,data,src,dst,length);
}
