/****************************************************************************/
/* IPMON version 1.0                                                        */
/****************************************************************************/
/* You can redistribute this program as much as you want.  All I ask is     */
/* that you give credit where credit is due.  If you make modifications or  */
/* bug fixes, please send them to me so they can be incorporated into the   */
/* next release.                                                            */
/*                                                                          */
/* IPMON takes its basic packet processing shell from NFSWATCH.  Much       */
/* thanks to the creators of NFSWATCH:                                      */
/*                                                                          */
/* Dave Curry					Jeff Mogul                  */
/* SRI International				Digital Equipment Corp.     */
/* 333 Ravenswood Avenue			Western Research Laboratory */
/* Menlo Park, CA 94025				100 Hamilton Avenue         */
/* davy@erg.sri.com				Palo Alto, CA 94301         */
/* 						mogul@decwrl.dec.com        */
/*                                                                          */
/* For a good set of base code for me to hack and build on.                 */
/*                                                                          */
/* Randall Stewart                                                          */
/* Fujitsu Network Switching                                                */
/* 4403 Bland Road                                                          */
/* Raleigh, N.C. 27609                                                      */
/* rrs%fns-nc1%mcnc-ort@uunet.uu.net                                        */
/****************************************************************************/
#include <sys/time.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <net/if.h>
#include <net/nit_if.h>
#include <net/nit_buf.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>
#include <netinet/ip_var.h>
#include <netinet/udp_var.h>
#include <netinet/tcpip.h>
#ifdef sun
#  define NND	1
#  include <sun/ndio.h>
#endif /* sun */
#include <curses.h>
#include <stdio.h>
#include <signal.h>

#include "ipmon.h" 
#include "externs.h"
#include "screen.h"

static int yposLast;

static int scanStart=0;
static int scanPoint=0;
static int scanEnd=0;
static int scanLast=0;

/*
 * Screen text items which are always displayed.
 */
static struct scrtxt scrtxts[] = {

	SCR_HOST_X,	SCR_HOST_Y,	dsthost,
	51,1,"Broadcast",
	51,2,"NetDisk  ",
	0,3,"Arp ",
	13,3,"R-Arp",
	30,3,"Unknown",
	50,3,"IP-?",
	64,3,"Icmp",
	0,0,"Total Lost (mon)",
	0,SCR_IF_Y, "BandWidth",
	SCR_ELAPS_X0,	SCR_ELAPS_Y,	"Elapsed time:",
	SCR_ELAPS_X,	SCR_ELAPS_Y,	"00:00:00",
	SCR_PKTINT_X0,	SCR_PKTINT_Y,	"Interval packets:",
	SCR_PKTTOT_X0,	SCR_PKTTOT_Y,	"Total packets:",
	SCR_PKTHDR_X,	SCR_PKTHDR_Y,	"int       total",
	SCR_PKTHDR_X+SCR_MIDDLE,	SCR_PKTHDR_Y,
					"int       total",
	0,SCR_PKTHDR_Y,"UDP Totals",
	SCR_MIDDLE,SCR_PKTHDR_Y,"TCP Totals",
	0,16,"UDP Total",
	SCR_MIDDLE,16,"TCP Total",
	-1,		-1,		NULL
};
static int helpUp=0;

static struct scrtxt scrhelp[] = {
0,CHILDS_Y,"d [t|u] all                    - delete all tcp/udp totals from display",
0,(CHILDS_Y+1),"d [t|u|i] port#                - delete tcp/udp/icmp port from totals display",
0,(CHILDS_Y+2),"a [t|u|i] port#                - add tcp/udp/icmp port to totals display",
0,(CHILDS_Y+3),"m [t|u|off|unknown|unknownip] port# - monitor data on/off for port#",
0,(CHILDS_Y+4),"show [totals|int]              - toggel totals to interval/totals",
0,(CHILDS_Y+5),"log [off|what|open] <filename> - log file control off,open and what log",
0,(CHILDS_Y+6),"flush/silent/verbose           - flush log/silent mode/verbose mode",
0,(CHILDS_Y+7),"scan port# - port#  - scan tcp/udp totals port # to port #",
-1,-1,NULL
};

/*
 * setup_screen - initialize the display screen.
 */
static char *tstr;

void
setup_screen(device)
char *device;
{
	void label_screen();
	int len;

	char *ctime();
	char buf[BUFSIZ];
	struct timeval tv;
	register struct scrtxt *st;

	/*
	 * Initialize the screen.
	 */

	(void) initscr();
	screen_inited = 1;

	/*
	 * Turn echo off, cbreak on.
	 */
	(void) noecho();

	(void) cbreak();

	/*
	 * Clear the screen.
	 */
	(void) clear();

	/*
	 * Get the time of day.
 	 */
	(void) gettimeofday(&tv, (struct timeval *) 0);
	tstr = ctime(&tv.tv_sec);

	(void) mvprintw(SCR_DATE_Y, SCR_DATE_X, "%.24s", tstr);

	/*
	 * Now draw the various strings on the screen.
	 */
	for (st = scrtxts; st->s_text != NULL; st++)
		(void) mvprintw(st->s_y, st->s_x, "%s", st->s_text);

	showTotalLost();
	(void) mvprintw(SCR_PROMPT_Y, SCR_PROMPT_X0, prompt);
	label_screen();
	yposLast = CHILDS_Y;
	move(SCR_PROMPT_Y,(strlen(prompt) + 2 + strlen(inputBuf)));
	myrefresh();
}

void
reDraw()
{
	register struct scrtxt *st;
	void update_screen();
	int len;
	char buf[BUFSIZ];
	(void) mvprintw(SCR_DATE_Y, SCR_DATE_X, "%.24s", tstr);

	/*
	 * Now draw the various strings on the screen.
	 */
	(void) clear();
	for (st = scrtxts; st->s_text != NULL; st++)
		(void) mvprintw(st->s_y, st->s_x, "%s", st->s_text);

	(void) sprintf(buf, "Monitoring packets from all interfaces");

	len = strlen(buf);

	(void) mvprintw(SCR_IF_Y, SCR_MIDDLE - (len/2), buf);
	(void) mvprintw(SCR_PROMPT_Y, SCR_PROMPT_X0, prompt);
	label_screen();
	yposLast = CHILDS_Y;
	move(SCR_PROMPT_Y,(strlen(prompt) + 2 + strlen(inputBuf)));
	label_screen();
	update_screen();

	myrefresh();
	return;
}

/*
 * label_screen - put all packet counter labels on screen.
 */

void
label_screen()
{
	register int i;
	for(i=0;i<10;i++){
		if(pktTcp[i].pc_name == NULL){
			mvprintw(pktTcp[i].pc_namey,
			pktTcp[i].pc_namex,
			"                    ");
		}else{
			mvprintw(pktTcp[i].pc_namey,
			pktTcp[i].pc_namex,
			pktTcp[i].pc_name);
		}
		if(pktUdp[i].pc_name == NULL){
			mvprintw(pktUdp[i].pc_namey,
			pktUdp[i].pc_namex,
			"                    ");
		}else{
			mvprintw(pktUdp[i].pc_namey,
			pktUdp[i].pc_namex,
			pktUdp[i].pc_name);


		}
	}

	return;
}

/*
 * update_screen - update the screen with new information.
 */

void
update_screen()
{
	char *tstr;
	char *ctime();
	long now;
	struct timeval tv;
	double bandWidth,bandWidth6,bandWidth60,tenMegaBitByte;
	register int i,t1,t2,t3;

	(void) gettimeofday(&tv, (struct timezone *) 0);
	now = tv.tv_sec;
	(void) mvprintw(SCR_DATE_Y, SCR_DATE_X, "%.24s", ctime(&tv.tv_sec));

	tv.tv_sec -= starttime.tv_sec;
	tstr = prtime(tv.tv_sec);

	/*
	 * Print the headers.
	 */
	showTotalLost();
	(void) mvprintw(SCR_ELAPS_Y, SCR_ELAPS_X, "%.8s", tstr);
	(void) mvprintw(SCR_PKTINT_Y, SCR_PKTINT_X,
		"%8d %8d (dropped)",
		pkt_total[INTERVAL], pkt_drops[INTERVAL]);
	(void) mvprintw(SCR_PKTTOT_Y, SCR_PKTTOT_X,
		"%8d %8d (dropped)",
		pkt_total[SUMTOTAL], pkt_drops[SUMTOTAL]);
	(void) mvprintw(1, 64,"%8d",broadcast[(*showTotals)]);
	(void) mvprintw(2, 64,"%8d",netdisk[(*showTotals)]);
	(void) mvprintw(3,4,"%8d",arpCnts[(*showTotals)]);
	(void) mvprintw(3,20,"%8d",revarpCnts[(*showTotals)]);
	(void) mvprintw(3,40,"%8d",unknown[(*showTotals)]);
	(void) mvprintw(3,55,"%8d",unknownip[(*showTotals)]);
	(void) mvprintw(3,68,"%8d",icmpTotal[(*showTotals)]);
	if(statistics){
	t1 = (now - timeFlush[0]);
	if(t1 <= 0)
		t1 = 1;
	t2 = (now - timeFlush[1]);
	if(t2 <= 0)
		t2 = 1;
	t3 = (now - timeFlush[2]);
	if(t3 <= 0)
		t3 = 1;

	tenMegaBitByte = ((double)(1024 * 1024 * 10)) /((double)8.0);
	bandWidth6 = (((double)byteTotals[0]/(double)t1) / tenMegaBitByte) * 100.0;
	bandWidth60 = (((double)byteTotals[1]/(double)t2) / tenMegaBitByte) * 100.0;
	bandWidth = (((double)byteTotals[2]/(double)t3) / tenMegaBitByte) * 100.0;

	(void) mvprintw(SCR_IF_Y,13,"%8d %4.1f(%%) %8d %4.1f(%% cycle) %8d %4.1f(%% 6 cycles)"
		,byteTotals[2],bandWidth,
		byteTotals[0],bandWidth6,
		byteTotals[1],bandWidth60);
	}
	for(i=0;i<10;i++){
		mvprintw(pktTcp[i].pc_inty,
		pktTcp[i].pc_intx,
		"          ");

		mvprintw(pktTcp[i].pc_toty,
		pktTcp[i].pc_totx,
		"          ");

		mvprintw(pktUdp[i].pc_inty,
		pktUdp[i].pc_intx,
		"          ");

		mvprintw(pktUdp[i].pc_toty,
		pktUdp[i].pc_totx,
		"          ");

		if(pktUdp[i].pc_name != NULL){
			mvprintw(pktUdp[i].pc_inty,
			pktUdp[i].pc_intx,
			"%d",pktUdp[i].pc_total[INTERVAL]);
			mvprintw(pktUdp[i].pc_toty,
			pktUdp[i].pc_totx,
			"%d",pktUdp[i].pc_total[SUMTOTAL]);
		}
		if(pktTcp[i].pc_name != NULL){
			mvprintw(pktTcp[i].pc_inty,
			pktTcp[i].pc_intx,
			"%d",pktTcp[i].pc_total[INTERVAL]);
			mvprintw(pktTcp[i].pc_toty,
			pktTcp[i].pc_totx,
			"%d",pktTcp[i].pc_total[SUMTOTAL]);
		}
	}
	/* blank all tcp/udp totals */
	mvprintw(16,20,"         ");
	mvprintw(16,30,"         ");
	mvprintw(16,60,"         ");
	mvprintw(16,70,"         ");

	mvprintw(16,20,"%d",udpTotal[INTERVAL]);
	mvprintw(16,30,"%d",udpTotal[SUMTOTAL]);
	mvprintw(16,60,"%d",tcpTotal[INTERVAL]);
	mvprintw(16,70,"%d",tcpTotal[SUMTOTAL]);
	
	move(SCR_PROMPT_Y,(strlen(prompt) + 2 + strlen(inputBuf)));
	(void) myrefresh();

}

void
command()
{
	register int c,len;


	if ((c = getch()) == EOF)
		finish(-1);
	len = strlen(inputBuf);
	c &= 0177;
	switch(c){
		case 'q':
		case 'Q':
			finish(0);
			break;
		case 0xc:	/* cntl-l */
			move(0,0);
			clrtobot();
			myrefresh();
			kill(fk,SIGUSR2);
			sleep(1);
			(void) mvprintw(SCR_PROMPT_Y, SCR_PROMPT_X0, prompt);
			break;
		case '?':
			showHelp();
			break;
		case  '@':	/* for those att types */
		case 0x15:	/* cntl-u */
			inputBuf[0] = 0;
			move(SCR_PROMPT_Y,(strlen(prompt) + 2));
			clrtoeol();
			break;
		case  '\n':
			processCommand();
			mvprintw(SCR_PROMPT_Y,(strlen(prompt) + 2),"                                        ");
			move(SCR_PROMPT_Y,(strlen(prompt) + 2));
			inputBuf[0] = 0;
			break;
		case 0x7f:	/* DEL */
		case 0x8:	/* backspace */
		case '#':	/* for those old att types */
			if(len == 0)
				break;
			len--;
			inputBuf[len] = 0;
			move(SCR_PROMPT_Y,(strlen(prompt) + 2 + len));
			clrtoeol();
			break;
		default:
			inputBuf[(len+1)] = 0;
			inputBuf[len] = c;
			break;
	}
	mvprintw(SCR_PROMPT_Y,(strlen(prompt) + 2),"%s",inputBuf);
	myrefresh();
}


void
showError(str,str2)
char *str,*str2;
{
	int x,y;

	getyx(stdscr,y,x);
	move((LINES-2),0);
	clrtoeol();
	if(str2 == NULL)
		mvprintw((LINES-2),0,"%s",str);
	else
		mvprintw((LINES-2),0,"%s:%s",str,str2);
	move(y,x);
	myrefresh();
}

extern WINDOW *stdscr;
void
clearError()
{
	int x,y;

	getyx(stdscr,y,x);
	move((LINES-2),0);
	clrtoeol();	
	move(y,x);
}
void
processCommand()
{
	register int i,port;

	if(strncmp(inputBuf,"d t all",7) == 0){
		/* delete tcp port from display */
		for(i=0;i<10;i++){
			pktTcp[i].pc_name = NULL;
			pktTcp[i].port = -1;
		}
		showError("All Tcp Port Removed",NULL);
		return;
	}else if(strncmp(inputBuf,"d u all",7) == 0){
		/* delete udp port from display */
		for(i=0;i<10;i++){
			pktUdp[i].pc_name = NULL;
			pktUdp[i].port = -1;

		}
		showError("All Udp Port Removed",NULL);
		return;
	}else if(strncmp(inputBuf,"d t ",4) == 0){
		/* delete tcp port from display */
		port = strtol(&inputBuf[4],NULL,0);
		if(port > MAXNUMOFPORTS){
			showError("Port Number To BIG!",NULL);
			return;
		}
		for(i=0;i<10;i++){
			if(pktTcp[i].port == port){
				showError("Tcp Port Removed",pktTcp[i].pc_name);
				pktTcp[i].pc_name = NULL;
				pktTcp[i].port = -1;
				return;
			}
		}
	}else if(strncmp(inputBuf,"d i ",4) == 0){
		/* delete tcp port from display */
		port = strtol(&inputBuf[4],NULL,0);
		if(port > MAXICMPMSG){
			showError("Msg Number To BIG!",NULL);
			return;
		}
		port += MAXNUMOFPORTS;
		for(i=0;i<10;i++){
			if(pktTcp[i].port == port){
				showError("Icmp Msg Removed",pktTcp[i].pc_name);
				pktTcp[i].pc_name = NULL;
				pktTcp[i].port = -1;
				return;
			}
		}
	}else if(strncmp(inputBuf,"d u ",4) == 0){
		/* delete udp port from display */
		port = strtol(&inputBuf[4],NULL,0);
		if(port > MAXNUMOFPORTS){
			showError("Port Number To BIG!",NULL);
			return;
		}
		for(i=0;i<10;i++){
			if(pktUdp[i].port == port){
				showError("Udp Port Removed",pktUdp[i].pc_name);
				pktUdp[i].pc_name = NULL;
				pktUdp[i].port = -1;

				return;
			}
		}
	}else if(strncmp(inputBuf,"a u ",4) == 0){
		/* delete udp port from display */
		port = strtol(&inputBuf[4],NULL,0);
		if((port > MAXNUMOFPORTS) || (port < 0)){
			showError("Port Number To BIG!",NULL);
			return;
		}
		for(i=0;i<10;i++){
			if(pktUdp[i].pc_name == NULL){
				pktUdp[i].port = port;
				pktUdp[i].pc_total = udpCnts[port];
				pktUdp[i].pc_name = genString("UDP Port[",port,i);
				showError("Udp Port Added",pktUdp[i].pc_name);
				return;
			}
		}
		showError("No Room delete a UDP port First",NULL);
	}else if(strncmp(inputBuf,"a t ",4) == 0){
		/* delete udp port from display */
		port = strtol(&inputBuf[4],NULL,0);
		if((port > MAXNUMOFPORTS) || (port < 0)){
			showError("Port Number To BIG!",NULL);
			return;
		}
		for(i=0;i<10;i++){
			if(pktTcp[i].pc_name == NULL){
				pktTcp[i].port = port;
				pktTcp[i].pc_total = tcpCnts[port];
				pktTcp[i].pc_name = genString("TCP Port[",port,(i+10));
				showError("Tcp Port Added",pktTcp[i].pc_name);
				return;
			}
		}
		showError("No Room delete a TCP port First",NULL);
	}else if(strncmp(inputBuf,"a i ",4) == 0){
		/* delete udp port from display */
		port = strtol(&inputBuf[4],NULL,0);
		if((port > MAXICMPMSG) || (port < 0)){
			showError("Icmp Number To BIG!",NULL);
			return;
		}
		for(i=0;i<10;i++){
			if(pktTcp[i].pc_name == NULL){
				pktTcp[i].port = port + MAXNUMOFPORTS;
				pktTcp[i].pc_total = icmpCnts[port];
				pktTcp[i].pc_name = genString("Icmp Msg[",port,(i+10));
				showError("Icmp Message Added",pktTcp[i].pc_name);
				return;
			}
		}
		showError("No Room delete a TCP port First",NULL);
	}else if (strncmp(inputBuf,"m u ",4) == 0){
		*packetOfInterest = strtol(&inputBuf[4],NULL,0);
		if((*packetOfInterest > MAXNUMOFPORTS) || (*packetOfInterest < 0)){
			*packetOfInterest = -1;
			showError("Port Number To BIG!",NULL);
			return;
		}
		*tcpFlag = UDPVALUE;
		*totMonLost = 0;
		showError("UDP monitor of port set",NULL);
	}else if (strncmp(inputBuf,"m t ",4) == 0){
		*packetOfInterest = strtol(&inputBuf[4],NULL,0);
		if((*packetOfInterest > MAXNUMOFPORTS) || (*packetOfInterest < 0)){
			*packetOfInterest = -1;
			showError("Port Number To BIG!",NULL);
			return;
		}
		*tcpFlag = TCPVALUE;
		*totMonLost = 0;
		showError("TCP monitor of port set",NULL);
	}else if (strncmp(inputBuf,"m i ",4) == 0){
		*packetOfInterest = strtol(&inputBuf[4],NULL,0);
		if((*packetOfInterest > 19) || (*packetOfInterest < 0)){
			*packetOfInterest = -1;
			showError("Icmptype to big/small",NULL);
			return;
		}
		*tcpFlag = ICMPVALUE;
		*totMonLost = 0;
		showError("ICMP monitor of port set",NULL);
	}else if (strncmp(inputBuf,"m revarp",9) == 0){
		*tcpFlag = REVARP_VALUE;
		showError("Reverse Arp on",NULL);
	}else if (strncmp(inputBuf,"m unknownip",9) == 0){
		*tcpFlag = UNKNOWNIP_VALUE;
		showError("Unknown IP packet monitor turned on",NULL);
	}else if (strncmp(inputBuf,"m unknown",9) == 0){
		*tcpFlag = UNKNOWN_VALUE;
		showError("Unknown packet monitor turned on",NULL);
	}else if (strncmp(inputBuf,"silent",6) == 0){
		silent = 1;
		showError("silent mode activated",NULL);
	}else if (strncmp(inputBuf,"verbose",7) == 0){
		silent = 0;
		showError("verbose mode activated",NULL);
	}else if (strncmp(inputBuf,"show totals",11) == 0){
		*showTotals = SUMTOTAL;
	}else if (strncmp(inputBuf,"show int",8) == 0){
		*showTotals = INTERVAL;
	}else if(strncmp(inputBuf,"flush",5) == 0){
		if(logfp != NULL){
			fflush(logfp);
			showError("Log file flushed",NULL);
		}else
			showError("Log not opened",NULL);
	}else if (strncmp(inputBuf,"m off",5) == 0){
		int yy;

		*tcpFlag = -1;
		*packetOfInterest = -1;
		*totMonLost = 0;
		for(yy=CHILDS_Y;yy<=yposLast;yy++){
			move(yy,0);
			clrtoeol();
		}
		yposLast = CHILDS_Y;
		showError("monitor terminated",NULL);
	}else if (strncmp(inputBuf,"view",4) == 0){
		viewFile(&inputBuf[4]);
	}else if (strncmp(inputBuf,"log off",7) == 0){
		fclose(logfp);
		strcpy(logFileName,"None");
		logfp = NULL;
		showError("Log is Now Closed",NULL);
	}else if (strncmp(inputBuf,"log what ",8) == 0){
		showError("log open to",logFileName);
	}else if (strncmp(inputBuf,"log open ",9) == 0){
		if(logfp != NULL){
			fclose(logfp);
			logfp = NULL;
		}
		logfp = fopen(&inputBuf[9],"a+");
		if(logfp == NULL){
			showError("Can't open log file",NULL);
			strcpy(logFileName,"None");
		}
		strcpy(logFileName,&inputBuf[9]);
		showError("Log Open",logFileName);
	}else if(strncmp(inputBuf,"scan off",8) == 0){
		struct itimerval itv;
		/* stop any running timer */
		(void) bzero((char *) &itv, sizeof(struct itimerval));
		(void) setitimer(ITIMER_REAL, &itv, (struct itimerval *)0);
		scanStart = scanEnd = 0;
		showError("scan Terminated",NULL);
	}else if(strncmp(inputBuf,"scan ",4) == 0){
		struct itimerval itv;
		char *end,*to;
		int lenn,ii;
		/* stop any running timer */
		(void) bzero((char *) &itv, sizeof(struct itimerval));
		(void) setitimer(ITIMER_REAL, &itv, (struct itimerval *)0);

		scanStart = strtol(&inputBuf[5],&end,0);
		if(end == &inputBuf[5])
			goto syntax;
		lenn = strlen(end);
		for(ii=0;ii<lenn;ii++){
			if(end[ii] == '-'){
				ii++;
				break;
			}
		}
		if(ii >= lenn){
			syntax:
			showError("Invalid Syntax use ->scan port# - port#",NULL);
			return;
		}
		scanEnd = strtol(&end[ii],&end,0);
		if((scanEnd < 0) || (scanEnd > MAXNUMOFPORTS)){
			showError("End port of Scan invalid",NULL);
			return;
		}
		if((scanStart < 0) || (scanStart > MAXNUMOFPORTS)){
			showError("Beginning port of Scan invalid",NULL);
			return;
		}
		scanPoint = scanStart;
		if((scanEnd - scanStart) <= 10){
			showError("Activating scan once only - all will display",NULL);
			scanLoop();
			scanEnd = scanStart = scanPoint = 0;
			return;
		}
		scanLoop();
		/* start the scan */
		itv.it_interval.tv_sec = cycletime * scanCycle;
		itv.it_interval.tv_usec = 0;
		itv.it_value = itv.it_interval;
		(void) setitimer(ITIMER_REAL, &itv, (struct itimerval *) 0);
		showError("Scan initiated",NULL);
	}else{
		showError("Unknown command",NULL);
	}
	return;
}

static long bigBuf[(NIT_CHUNKSIZE/sizeof(long))];

void
displayPkts(bytes)
int bytes;
{
	register char *bp, *cp, *bufstop;
	struct nit_bufhdr *hdrp;
	struct nit_ifdrops *ndp;
	static char *bufAlign=NULL;
	char localbuf[64];

	if(bufAlign == NULL){
		bufAlign = (char *)bigBuf;
	}
	memcpy(bufAlign,bufInTail,bytes);
	bufstop = (char *)((int)(bufAlign) + bytes);
	bp = bufAlign;
	bufInTail += NIT_CHUNKSIZE;
	if(bufInTail >= bufInEND)
		bufInTail = bufInTOP;
	while (bp < bufstop) {
		cp = bp;
		/*
		 * Get the nit header.
		 */
		hdrp = (struct nit_bufhdr *) cp;
		if((long)hdrp % 4){
			showError("Buffer overrun/alignment - Mangled buffer",NULL);
			if(logfp != NULL){
				fprintf(logfp,"Buffer OVERRUN! - Mangled buffer\n\n");
			}
			break;
		}
		if(hdrp->nhb_msglen > NIT_CHUNKSIZE){
			showError("Buffer0 overrun - Mangled buffer",NULL);
			if(logfp != NULL){
				fprintf(logfp,"Buffer OVERRUN! - Mangled buffer\n\n");
			}
			break;
		}
		if(hdrp->nhb_totlen > NIT_CHUNKSIZE){
			showError("Buffer1 overrun - Mangled buffer",NULL);
			if(logfp != NULL){
				fprintf(logfp,"Buffer OVERRUN! - Mangled buffer\n\n");
			}
			break;
		}
		if(!hdrp->nhb_msglen){
			showError("Buffer2 overrun - Mangled buffer",NULL);
			if(logfp != NULL){
				fprintf(logfp,"Buffer OVERRUN! - Mangled buffer\n\n");
			}
			break;
		}
		if(!hdrp->nhb_totlen){
			showError("Buffer3 overrun - Mangled buffer",NULL);
			if(logfp != NULL){
				fprintf(logfp,"Buffer OVERRUN! - Mangled buffer\n\n");
			}
			break;
		}
		if(((int)(cp) + hdrp->nhb_msglen) > (int)(bufstop)){
			showError("Buffer4 overrun",NULL);
			if(logfp != NULL){
				fprintf(logfp,"Buffer OVERRUN!\n\n");
			}
			break;
		}
		cp += sizeof(struct nit_bufhdr);

		/*
		 * Get the number of dropped packets.
		 */
		ndp = (struct nit_ifdrops *) cp;
		cp += sizeof(struct nit_ifdrops);

		/*
		 * Filter the packet.
		 */
		showPkt(cp, hdrp->nhb_msglen);

		/*
		 * Skip over this packet.
		 */
		bp += hdrp->nhb_totlen;
	}
	return;
}

void
showIcmp(icp, length, src, dst,eheader)
register struct icmp *icp;
u_long src, dst;
u_int length;
struct ether_header *eheader;
{
	int y,x,yy;

        if(icp->icmp_type == *packetOfInterest){
		getyx(stdscr,y,x);
		showEnet(eheader->ether_type,eheader->ether_dhost.ether_addr_octet,
			eheader->ether_shost.ether_addr_octet,(char *)icp,0);
		showAddress(src,dst);
		if(!silent){
			if((yposLast) >= (LINES-3)){
				/* next update won't fit */
				/* clear and start at top */
				for(yy=CHILDS_Y;yy<=yposLast;yy++){
					move(yy,0);
					clrtoeol();
				}
				yposLast = CHILDS_Y;
			}
			mvprintw(yposLast++,0,
				"ICMP MSG %d\n",icp->icmp_type);
		}
		if(logfp != NULL){
			fprintf(logfp,"ICMP MSG %d\n",icp->icmp_type);
		}
	}
}

void
showTcp(tcp, length, src, dst)
register struct tcphdr *tcp;
u_long src, dst;
u_int length;
{
	register int i,j;
	int x,y,yy;
	/*
	 * Display the TCP packet ?
	 */
        if((tcp->th_dport == *packetOfInterest) ||
        (tcp->th_sport == *packetOfInterest)){
		getyx(stdscr,y,x);
		showAddress(src,dst);
		if((yposLast) >= (LINES-3)){
			/* next update won't fit */
			/* clear and start at top */
			for(yy=CHILDS_Y;yy<=yposLast;yy++){
				move(yy,0);
				clrtoeol();
			}
			yposLast = CHILDS_Y;
		}
		if(logfp != NULL){
			unsigned char *data;
			int len;
			data = (unsigned char *)((int)tcp + (tcp->th_off * 4));
			fprintf(logfp,
			"Port(from) %d Port(to) %d Seq=0x%x Ack=0x%x\n",
			tcp->th_sport,tcp->th_dport,
			tcp->th_seq,tcp->th_ack);
			fprintf(logfp,
			"off=0x%x win=0x%x sum=0x%x urp=0x%x th_flags=0x%x\n",
			tcp->th_off,tcp->th_win,
			tcp->th_sum,tcp->th_urp,
			tcp->th_flags);
			len = length - (tcp->th_off * 4);
			for(i=0,j=0;i<len;i++){
				fprintf(logfp,"%2.2x ",data[i]);
				if(((i+1) % 16) == 0){
					/* generate ascii and eol */
					fprintf(logfp,"  ");
					for(j=((i+1)-16);j<=i;j++){
						if((data[j] >= 040) && (data[j] <= 0176))
							fprintf(logfp,"%c",data[j]);
						else
							fprintf(logfp,".",data[j]);
					}
					fprintf(logfp,"\n");
				}
			}
			{
				int dist,sp;
				dist = ((16 - (i - j)) * 3) + 2;
				for(sp=0;sp<dist;sp++)
					fprintf(logfp," ");
			}
			for(;j<i;j++){
				if((data[j] >= 040) && (data[j] <= 0176))
					fprintf(logfp,"%c",data[j]);
				else
					fprintf(logfp,".",data[j]);
			}
			fprintf(logfp,"\n\n");
		}

		if(!silent){
			mvprintw(yposLast++,0,
			"Port(from) %d Port(to) %d Seq=0x%x Ack=0x%x",
			tcp->th_sport,tcp->th_dport,
			tcp->th_seq,tcp->th_ack);

			if((yposLast) >= (LINES-3)){
				/* next update won't fit */
				/* clear and start at top */
				for(yy=CHILDS_Y;yy<=yposLast;yy++){
					move(yy,0);
					clrtoeol();
				}
				yposLast = CHILDS_Y;
			}
			mvprintw(yposLast++,0,
			"off=0x%x win=0x%x sum=0x%x urp=0x%x",
			tcp->th_off,tcp->th_win,
			tcp->th_sum,tcp->th_urp);

			if((yposLast) >= (LINES-3)){
				/* next update won't fit */
				/* clear and start at top */
				for(yy=CHILDS_Y;yy<=yposLast;yy++){
					move(yy,0);
					clrtoeol();
				}
				yposLast = CHILDS_Y;
			}
		}
		move(y,x);
		myrefresh();
	}
}

void
showUdp(udp, length, src, dst)
register struct udphdr *udp;
u_long src, dst;
u_int length;
{
	register int i,j;
	int x,y,yy;
	/*
	 * Display the UDP packet ?
	 */
	if((udp->uh_dport == *packetOfInterest) ||
	(udp->uh_sport == *packetOfInterest)){
		getyx(stdscr,y,x);
		showAddress(src,dst);
		if((yposLast) >= (LINES-3)){
			/* next update won't fit */
			/* clear and start at top */
			for(yy=CHILDS_Y;yy<=yposLast;yy++){
				move(yy,0);
				clrtoeol();
			}
			yposLast = CHILDS_Y;
		}
		if(logfp != NULL){
			int len;
			unsigned char *data;
			data = (unsigned char *)((int)(udp) + sizeof(struct udphdr));
			len = length - sizeof(struct udphdr);

			fprintf(logfp,
			"Port(from) %d Port(to) %d Length=0x%x Sum=0x%x\n",
			udp->uh_sport,udp->uh_dport,
			udp->uh_ulen,udp->uh_sum);

			for(i=0,j=0;i<len;i++){
				fprintf(logfp,"%2.2x ",data[i]);
				if(((i+1) % 16) == 0){
					/* generate ascii and eol */
					fprintf(logfp,"  ");
					for(j=((i+1)-16);j<=i;j++){
						if((data[j] >= 040) && (data[j] <= 0176))
							fprintf(logfp,"%c",data[j]);
						else
							fprintf(logfp,".",data[j]);
					}
					fprintf(logfp,"\n");
				}
			}
			{
				int dist,sp;
				dist = ((16 - (i - j)) * 3) + 2;
				for(sp=0;sp<dist;sp++)
					fprintf(logfp," ");
			}
			for(;j<i;j++){
				if((data[j] >= 040) && (data[j] <= 0176))
					fprintf(logfp,"%c",data[j]);
				else
					fprintf(logfp,".",data[j]);
			}
			fprintf(logfp,"\n\n");
		}
		if(!silent){
			mvprintw(yposLast++,0,
			"Port(from) %d Port(to) %d Length=0x%x Sum=0x%x",
			udp->uh_sport,udp->uh_dport,
			udp->uh_ulen,udp->uh_sum);

			if((yposLast) >= (LINES-3)){
				/* next update won't fit */
				/* clear and start at top */
				for(yy=CHILDS_Y;yy<=yposLast;yy++){
					move(yy,0);
					clrtoeol();
				}
				yposLast = CHILDS_Y;
			}
		}
		move(y,x);
		myrefresh();

	}
}

void
showAddress(src,dst)
u_long src, dst;
{
	unsigned char source[4],destination[4];
	source[0] = ((src & 0xff000000) >> 24);
	source[1] = ((src & 0x00ff0000) >> 16);
	source[2] = ((src & 0x0000ff00) >> 8);
	source[3] = (src & 0x000000ff);
	destination[0] = ((dst & 0xff000000) >> 24);
	destination[1] = ((dst & 0x00ff0000) >> 16);
	destination[2] = ((dst & 0x0000ff00) >> 8);
	destination[3] = (dst & 0x000000ff);
	if(logfp != NULL){
		struct timeval v;

		gettimeofday(&v,NULL); 
		fprintf(logfp,"TimeStamp:0x%x:0x%x\n",
			v.tv_sec,v.tv_usec);
		fprintf(logfp,"IP(from) %d.%d.%d.%d IP(to) %d.%d.%d.%d\n",
			source[0],source[1],source[2],source[3],
			destination[0],destination[1],destination[2],destination[3]);
	}
	if(!silent)
		mvprintw(yposLast++,0,"IP(from) %d.%d.%d.%d IP(to) %d.%d.%d.%d",
		source[0],source[1],source[2],source[3],
		destination[0],destination[1],destination[2],destination[3]);
}

void
myrefresh()
{
	get_semy();
	refresh();
	put_semy();
}

void
clearHelp()
{
	int yy;

	if(helpUp){
		for(yy=CHILDS_Y;yy<=LINES-3;yy++){
			move(yy,0);
			clrtoeol();
		}
		helpUp = 0;
		myrefresh();
	}
}

void
showHelp()
{
	struct scrtxt *scp;
	int x,y;

	if((*tcpFlag != -1) && !silent){
		showError("Can't display help and have monitor up",NULL);
		return;
	}
	helpUp = 1;
	getyx(stdscr,y,x);
	for(scp = scrhelp;scp->s_text != NULL;scp++){
		mvprintw(scp->s_y,scp->s_x,"%s",scp->s_text);
	}
	move(y,x);
	myrefresh();
	return;
}



void
viewFile(file)
char *file;
{
	struct scrtxt *scp;
	int x,y,at,len,disp,i,c;
	char buf[256];
	FILE *ioFile;

	if(file[0] == ' ')
		file++;

	if((*tcpFlag != -1) && !silent){
		showError("Monitor is on Can't display file",file);
		return;
	}
	ioFile = fopen(file,"r");
	if(ioFile == NULL){
		showError("Can't open file:",file);
		return;
	}

	getyx(stdscr,y,x);

	/* clear off stuff */
	for(at=CHILDS_Y;at<=LINES-3;at++){
		move(at,0);
		clrtoeol();
	}
	/* setup counters */
	at = 0;
	disp = CHILDS_Y;
	len = (LINES-3) - CHILDS_Y;

	/* start the read stuff */
	while(fgets(buf,256,ioFile) != NULL){
		for(i=strlen(buf);i;i--){
			if(buf[i] == '\n'){
				buf[i] = 0;
				break;
			}
		}
		if(i> 75){
			buf[75] = 0;
		}
		mvprintw(disp++,0,"%s",buf);
		at++;
		if(at >= len){
			showError("Hit q to quit, anything else to continue",NULL);
			move(y,x);
			myrefresh();
			if ((c = getch()) == EOF)
				break;
			c &= 0177;
			if(c == 'q')
				break;
			for(at=CHILDS_Y;at<=LINES-3;at++){
				move(at,0);
				clrtoeol();
			}
			at = 0;
			disp = CHILDS_Y;
		}
	}
	fclose(ioFile);
	move(y,x);
	myrefresh();
	return;
}

void
scanLoop()
{
	register int i;

	if(!scanStart && !scanEnd)
		return;
	/* delete all ports */
	for(i=0;i<10;i++){
		pktTcp[i].pc_name = NULL;
		pktUdp[i].pc_name = NULL;
		pktUdp[i].port = -1;
		pktTcp[i].port = -1;

	}
	for(i=0;i<10;scanPoint++,i++){
		pktUdp[i].port = scanPoint;
		pktUdp[i].pc_total = udpCnts[scanPoint];
		pktUdp[i].pc_name = genString("UDP Port[",scanPoint,i);
		pktTcp[i].port = scanPoint;
		pktTcp[i].pc_total = tcpCnts[scanPoint];
		pktTcp[i].pc_name = genString("TCP Port[",scanPoint,(i+10));
		if(scanPoint >= scanEnd){
			scanPoint = scanStart;
			break;
		}
	}

}
void
showTotalLost()
{
	int x,y;
	getyx(stdscr,y,x);
	mvprintw(0,18,"%8d",*totMonLost);
	move(y,x);
}

void
showEnet(etype,dest,sour,packet,length)
u_short etype;
unsigned char *dest,*sour,*packet;
int length;
{
	register int i,j;
	int x,y,yy;
	/*
	 * Display the packet ?
	 */
	if(!silent){
		getyx(stdscr,y,x);
		mvprintw(yposLast++,0,
			"From %x.%x.%x.%x.%x.%x To %x.%x.%x.%x.%x.%x Length=0x%x",
			sour[0],sour[1],sour[2],sour[3],sour[4],sour[5],
			dest[0],dest[1],dest[2],dest[3],dest[4],dest[5],length);
	}
	if((yposLast) >= (LINES-3)){
		/* next update won't fit */
		/* clear and start at top */
		for(yy=CHILDS_Y;yy<=yposLast;yy++){
			move(yy,0);
			clrtoeol();
		}
		yposLast = CHILDS_Y;
	}
	if(logfp != NULL){
		int len;
		unsigned char *data;
		fprintf(logfp,"From %x.%x.%x.%x.%x.%x To %x.%x.%x.%x.%x.%x Length=0x%x\n",
			sour[0],sour[1],sour[2],sour[3],sour[4],sour[5],
			dest[0],dest[1],dest[2],dest[3],dest[4],dest[5],length);
		if(length == 0){
			if(!silent){
				move(y,x);
				myrefresh();
			}
			return;
		}
		data = packet;
		len = length;
		for(i=0,j=0;i<len;i++){
			fprintf(logfp,"%2.2x ",data[i]);
			if(((i+1) % 16) == 0){
				/* generate ascii and eol */
				fprintf(logfp,"  ");
				for(j=((i+1)-16);j<=i;j++){
					if((data[j] >= 040) && (data[j] <= 0176))
						fprintf(logfp,"%c",data[j]);
					else
						fprintf(logfp,".",data[j]);
				}
				fprintf(logfp,"\n");
			}
		}
		{
			int dist,sp;
			dist = ((16 - (i - j)) * 3) + 2;
			for(sp=0;sp<dist;sp++)
				fprintf(logfp," ");
		}
		for(;j<i;j++){
			if((data[j] >= 040) && (data[j] <= 0176))
				fprintf(logfp,"%c",data[j]);
			else
				fprintf(logfp,".",data[j]);
		}
		fprintf(logfp,"\n\n");
	}
	if(!silent){
		move(y,x);
		myrefresh();
	}
}

void
showJunk(ip_v,ip_hl,ip_tos,ip_id,ip_p,packet,src,dst,length)
int ip_v,ip_hl;
u_char ip_tos;
u_short ip_id;
u_char ip_p;
int *packet;
u_long src,dst;
u_int length;
{
	int y,x,i,j,yy;

	if(!silent){
		getyx(stdscr,y,x);
		showAddress(src,dst);
		mvprintw(yposLast++,0,
		"ip_v=0x%x,ip_hl=0x%x,ip_tos=0x%x,ip_p=0x%x",
		ip_v,ip_hl,ip_tos,ip_p);
	}
	if((yposLast) >= (LINES-3)){
		/* next update won't fit */
		/* clear and start at top */
		for(yy=CHILDS_Y;yy<=yposLast;yy++){
			move(yy,0);
			clrtoeol();
		}
		yposLast = CHILDS_Y;
	}
	if(logfp != NULL){
		int len;
		unsigned char *data;
		data = (unsigned char *)packet;
		len = length;

		fprintf(logfp,"ip_v=0x%x,ip_hl=0x%x,ip_tos=0x%x,ip_p=0x%x\n",
		ip_v,ip_hl,ip_tos,ip_p);
		for(i=0,j=0;i<len;i++){
			fprintf(logfp,"%2.2x ",data[i]);
			if(((i+1) % 16) == 0){
				/* generate ascii and eol */
				fprintf(logfp,"  ");
				for(j=((i+1)-16);j<=i;j++){
					if((data[j] >= 040) && (data[j] <= 0176))
						fprintf(logfp,"%c",data[j]);
					else
						fprintf(logfp,".",data[j]);
				}
				fprintf(logfp,"\n");
			}
		}
		{
			int dist,sp;
			dist = ((16 - (i - j)) * 3) + 2;
			for(sp=0;sp<dist;sp++)
				fprintf(logfp," ");
		}
		for(;j<i;j++){
			if((data[j] >= 040) && (data[j] <= 0176))
				fprintf(logfp,"%c",data[j]);
			else
				fprintf(logfp,".",data[j]);
		}
		fprintf(logfp,"\n\n");
	}
	if(!silent){
		move(y,x);
		myrefresh();
	}
}
