/****************************************************************************/
/* IPMON version 1.0                                                        */
/****************************************************************************/
/* You can redistribute this program as much as you want.  All I ask is     */
/* that you give credit where credit is due.  If you make modifications or  */
/* bug fixes, please send them to me so they can be incorporated into the   */
/* next release.                                                            */
/*                                                                          */
/* IPMON takes its basic packet processing shell from NFSWATCH.  Much       */
/* thanks to the creators of NFSWATCH:                                      */
/*                                                                          */
/* Dave Curry					Jeff Mogul                  */
/* SRI International				Digital Equipment Corp.     */
/* 333 Ravenswood Avenue			Western Research Laboratory */
/* Menlo Park, CA 94025				100 Hamilton Avenue         */
/* davy@erg.sri.com				Palo Alto, CA 94301         */
/* 						mogul@decwrl.dec.com        */
/*                                                                          */
/* For a good set of base code for me to hack and build on.                 */
/*                                                                          */
/* Randall Stewart                                                          */
/* Fujitsu Network Switching                                                */
/* 4403 Bland Road                                                          */
/* Raleigh, N.C. 27609                                                      */
/* rrs%fns-nc1%mcnc-ort@uunet.uu.net                                        */
/****************************************************************************/
#include <sys/param.h>
#include <sys/stat.h>
#include <curses.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>
#include <stdio.h>
#ifdef sun
#include <mntent.h>
#include <exportent.h>
#endif /* sun */

#include "ipmon.h"
#include "externs.h"
#include "screen.h"
static unsigned long cntOfClears=0;
/*
 * clear_vars - set interval counters to zero.
 */
void
clear_vars()
{
	register int i;
	cntOfClears++;

	pkt_drops[INTERVAL] = 0;
	pkt_total[INTERVAL] = 0;
	broadcast[INTERVAL] = 0;
	tcpTotal[INTERVAL] = 0;
	udpTotal[INTERVAL] = 0;
	icmpTotal[INTERVAL] = 0;
	arpCnts[INTERVAL]  = 0;
	revarpCnts[INTERVAL] = 0;
	unknown[INTERVAL] = 0;
	netdisk[INTERVAL] = 0;
	unknownip[INTERVAL] = 0;
	/* every cycle update bandwidth this sample */
	if(flogging != NULL){
		fprintf(flogging,"---------------\n");
		fprintf(flogging,"%d:%d:%d\n",byteTotals[0],
			byteTotals[1],byteTotals[2]);
		fprintf(flogging,"---------------\n");
	}
	timeFlush[0] = time(0);
	byteTotals[0] = 0;
	if((cntOfClears % 6) == 0){
		/* every 6 cycles clear bandwidth load */
		timeFlush[1] = time(0);
		byteTotals[1] = 0;
	}
	for(i=0;i<MAXNUMOFPORTS;i++){
		tcpCnts[i][INTERVAL] = 0;
		udpCnts[i][INTERVAL] = 0;
	}
	for(i=0;i<MAXICMPMSG;i++)
		icmpCnts[i][INTERVAL] = 0;

}

/*
 * savestr - save a string in dynamically allocated memory.
 */
char *
savestr(s)
register char *s;
{
	char *malloc();
	register char *str;

	str = malloc(strlen(s) + 1);

	if (str == NULL) {
		(void) fprintf(stderr, "%s: out of memory.\n", pname);
		finish(-1);
	}

	(void) strcpy(str, s);
	return(str);
}

char *
genString(s,x,i)
char *s;
int x;
int i;
{
	sprintf(strings[i],"%s%d]",s,x);
	return(strings[i]);
}

/*
 * prtime - convert a time to hh:mm:ss.
 */
char *
prtime(sec)
time_t sec;
{
	int hh, mm, ss;
	static char tbuf[16];

	hh = sec / 3600;
	sec %= 3600;

	mm = sec / 60;
	sec %= 60;

	ss = sec;

	(void) sprintf(tbuf, "%02d:%02d:%02d", hh, mm, ss);
	return(tbuf);
}

/*
 * error - print an error message preceded by the program name.
 */
void
error(str)
register char *str;
{
	char buf[BUFSIZ];

	(void) sprintf(buf, "%s: %s", pname, str);
	(void) perror(buf);
}

/*
 * finish - clean up and exit.
 */
void
finish(code)
int code;
{
	register int i;
	/*
	 * Close the nit device.
	 */
	if(fk > 0){
		/* kill child. */
		kill(fk,SIGUSR1);
	}
	for(i=0;i<ninterfaces;i++){
		(void) close(if_fd[i]);
	}
	if(logfp != NULL)
		fclose(logfp);
	/*
	 * End curses.
	 */
	if ((screen_inited) && (fk > 0)) {
		(void) nocbreak();
		(void) echo();
		(void) move(SCR_PROMPT_Y, SCR_PROMPT_X0);
		(void) clrtoeol();
		(void) myrefresh();
		(void) endwin();
	}
	if (code < 0)
		(void) exit(-code);

	(void) exit(0);
}

/*
 * setup_pkt_counters - set up packet counter screen coordinates.
 */
void
setup_pkt_counters()
{
	register int i;

	for(i=0;i<MAXNUMOFPORTS;i++){
		udpCnts[i][0] = udpCnts[i][1] = 0;
		tcpCnts[i][0] = tcpCnts[i][1] = 0;
	}
	for(i=0;i<MAXICMPMSG;i++)
		icmpCnts[i][0] = icmpCnts[i][1] = 0;

	tcpTotal[0] = tcpTotal[1] = 0;
	udpTotal[0] = udpTotal[1] = 0;
	broadcast[0] = broadcast[1] = 0;
	pkt_total[0] = pkt_total[1] = 0;
	unknownip[0] = unknownip[1] = 0;
	netdisk[0] = netdisk[1] = 0;
	arpCnts[0] = arpCnts[1] = 0;
	revarpCnts[0] = revarpCnts[1] = 0;
	unknown[0] = unknown[1] = 0;
	pkt_drops[0] = pkt_drops[1] = 0;

	for(i=0;i<10;i++){
		pktUdp[i].port = i+1;
		pktUdp[i].pc_total = udpCnts[i+1];
		pktUdp[i].pc_name  = genString("UDP Port[",i+1,i);
		pktUdp[i].pc_intx  = 20;
		pktUdp[i].pc_inty  = SCR_PKT_Y + i; 
		pktUdp[i].pc_totx  = 30;
		pktUdp[i].pc_toty  = SCR_PKT_Y + i;
		pktUdp[i].pc_namex = 0;
		pktUdp[i].pc_namey = SCR_PKT_Y + i;

		pktTcp[i].port = i+1;
		pktTcp[i].pc_total = tcpCnts[i+1];
		pktTcp[i].pc_name  = genString("TCP Port[",i+1,(i+10));
		pktTcp[i].pc_intx  = 60;
		pktTcp[i].pc_inty  = SCR_PKT_Y + i;
		pktTcp[i].pc_totx  = 70;
		pktTcp[i].pc_toty  = SCR_PKT_Y + i;
		pktTcp[i].pc_namex = SCR_MIDDLE;
		pktTcp[i].pc_namey = SCR_PKT_Y + i;

	}
}


/*
 * usage - print a usage message and exit.
 */

void
useage(me)
char *me;
{
	fprintf(stderr,"%s [-l logfile -c sec_per_cycle -s scanRate \n",me);
	finish(-1);
}

/*
 * wakeup - wake up for a screen update.
 */
void
wakeup()
{
	/*
	 * Re-sort and re-do the labels.
	 */
	if((*tcpFlag != -1) && (mumMode))
		return;
	label_screen();
	update_screen();
	clear_vars();
	if(flogging != NULL){
		fprintf(flogging,"-->%d:%d:%d<--\n",byteTotals[0],
			byteTotals[1],byteTotals[2]);
	}
}

void
setupShm()
{
	key_t key;
	long id;
	char *p,*baseaddr;
	int i;  
	int *kk;

	key = ftok("/etc/fstab",'D');
	id = shmget(key,((16+NUMOFBUFS) * NIT_CHUNKSIZE),IPC_CREAT|0666);
	if(id == -1){
		perror("Get of Shared Memory failed");
		exit(0);
	}
	baseaddr = (char *)shmat(id,0,0);
	kk = (int *)baseaddr;
	if(*kk == -1){
		perror("Attach of Shared Memory failed");
		exit(0);
	}
	p = (char *)((int)baseaddr + (NIT_CHUNKSIZE * NUMOFBUFS));
	foo = (char **)p;
	bufInEND = p;
	bufInHead = bufInTail =  bufInTOP = baseaddr;
	p += (sizeof(char *) * 5);
	packetOfInterest = (int *)p;
	p += sizeof(int *);
	tcpFlag = (int *)p;
	p += sizeof(int *);
	pktUdp = (PacketCounter *)p;
	p += (sizeof(PacketCounter) * 16);
	pktTcp = (PacketCounter *)p;
	p += (sizeof(PacketCounter) * 16);
	/* setup up no packets of interest */
	*packetOfInterest = -1;
	*tcpFlag = -1;
	totMonLost = (int *)p;
	*totMonLost = 0;
	p += sizeof(int);
	showTotals = (int *)p;
	p += sizeof(int);
	*showTotals = SUMTOTAL;
	/* allocate 64 bytes for all strings..	*/
	for(i=0;i<20;i++){
		strings[i] = p;
		p += (sizeof(char) * 64);
	}
	inputBuf = p;
	inputBuf[0] = 0;
	return;
}
